@@ -62,6 +62,7 @@ int main(int argc, char** argv)
 
   numpfx = 0;
   numsfx = 0;
+  fullstrip = 0;
 
   if (parse_aff_file(afflst)) {
     fprintf(stderr,"Error - in affix file loading\n");
@@ -82,7 +83,10 @@ int main(int argc, char** argv)
   }
 
   /* skip over the hash table size */
-  if (! fgets(ts, MAX_LN_LEN-1,wrdlst)) return 2;
+  if (! fgets(ts, MAX_LN_LEN-1,wrdlst)) {
+    fclose(wrdlst);
+    return 2;
+  }
   mychomp(ts);
 
   while (fgets(ts,MAX_LN_LEN-1,wrdlst)) {
@@ -139,6 +143,7 @@ int parse_aff_file(FILE * afflst)
        mychomp(line);
        ft = ' ';
        fprintf(stderr,"parsing line: %s\n",line);
+       if (strncmp(line,"FULLSTRIP",9) == 0) fullstrip = 1;
        if (strncmp(line,"PFX",3) == 0) ft = 'P';
        if (strncmp(line,"SFX",3) == 0) ft = 'S';
        if (ft != ' ') {
@@ -335,7 +340,7 @@ void pfx_add (const char * word, int len, struct affent* ep, int num)
     for (aent = ep, i = num; i > 0; aent++, i--) {
 
         /* now make sure all conditions match */        
-        if ((len > aent->stripl) && (len >= aent->numconds) &&
+        if ((len + fullstrip > aent->stripl) && (len >= aent->numconds) &&
             ((aent->stripl == 0) ||
             (strncmp(aent->strip, word, aent->stripl) == 0))) {
 
@@ -384,7 +389,7 @@ void suf_add (const char * word, int len, struct affent * ep, int num)
        * then strip off strip string and add suffix
        */
 
-      if ((len > aent->stripl) && (len >= aent->numconds) &&
+      if ((len + fullstrip > aent->stripl) && (len >= aent->numconds) &&
             ((aent->stripl == 0) ||
             (strcmp(aent->strip, word + len - aent->stripl) == 0))) {
 	cp = (unsigned char *) (word + len);
@@ -471,18 +476,20 @@ char * mystrsep(char ** stringp, const char delim)
       *stringp = dp+1;
       nc = (int)((unsigned long)dp - (unsigned long)mp);
       rv = (char *) malloc(nc+1);
-      memcpy(rv,mp,nc);
-      *(rv+nc) = '\0';
-      return rv;
+      if (rv) {
+        memcpy(rv,mp,nc);
+        *(rv+nc) = '\0';
+      }
     } else {
       rv = (char *) malloc(n+1);
-      memcpy(rv, mp, n);
-      *(rv+n) = '\0';
-      *stringp = mp + n;
-      return rv;
+      if (rv) {
+        memcpy(rv, mp, n);
+        *(rv+n) = '\0';
+        *stringp = mp + n;
+      }
     }
   }
-  return NULL;
+  return rv;
 }
 
 
@@ -490,9 +497,9 @@ char * mystrdup(const char * s)
 {
   char * d = NULL;
   if (s) {
-    int sl = strlen(s);
-    d = (char *) malloc(((sl+1) * sizeof(char)));
-    if (d) memcpy(d,s,((sl+1)*sizeof(char)));
+    int sl = strlen(s)+1;
+    d = (char *) malloc(sl);
+    if (d) memcpy(d,s,sl);
   }
   return d;
 }
